/*
**> In this program, we price call options using local volatility model with Backward and Forward Dupire PDE model.
** 
*/

new;
library option,pgraph;
OptionSet;

cls;

loadm databuf = Dupire-ImpVol;

Smile_K = vread(databuf,"K");
Smile_T = vread(databuf,"T");
Smile_ImpVol = vread(databuf,"ImpVol");

S0 = 100;
b = 0.05;
r = 0.05;

FileName = "Dupire11";
call LocalVol_Init(Smile_K,Smile_ImpVol,Smile_T,0,200,201,1,201,S0,b,r,FileName);

ImpVol_FileName = FileName $+ "-ImpVol";
ImpVol = loadd(ImpVol_FileName)';

let vectorTau = 0.01 0.01 0.01 0.20 0.20 0.20 0.5 0.5 0.5 1.0 1.0 1.0;
let vectorK =   95   100  105  95   100  105  95  100 105 95  100 105;
nIters = rows(vectorTau);

__output = 0;

i = 1;
do until i > nIters;

  print /flush "-------------------------------------------------------------------------------";

  Tau = vectorTau[i];
  K = vectorK[i];

  indx_K = FindIndex(_localVol_X,K);
  indx_Tau = FindIndex(_localVol_T,Tau);
  tmp_K = _localVol_X[indx_K];
  tmp_Tau = _localVol_T[indx_Tau];
  tmp_Sigma = ImpVol[indx_K,indx_Tau];
  C = EuropeanBS(S0,tmp_K,tmp_Sigma,tmp_Tau,b,r);
  print S0~tmp_K~tmp_Tau~tmp_Sigma~C;

  {tmp_S0,tmp_Tau,C} = LocalVol_Backward_PDE_Solve("call",K,0,0,tau,1.0);
  tmp_Sigma = EuropeanBS_ImpVol(tmp_S0,K,tmp_Tau,b,r,C);
  print tmp_S0~K~tmp_Tau~tmp_Sigma~C;

  {tmp_K,tmp_Tau,C} = LocalVol_Forward_PDE_Solve("call",K,0,0,tau,1.0);
  tmp_Sigma = EuropeanBS_ImpVol(S0,tmp_K,tmp_Tau,b,r,C);
  print S0~tmp_K~tmp_Tau~tmp_Sigma~C;
  
  i = i + 1;
endo;

/*
**> Remark that you may compute all the prices in one times with the Forward PDE model
**  It is faster !!!
*/

let K = 95 100 105;
let Tau = 0.01 0.20 0.5 1.0;

{tmp_K,tmp_Tau,C} = LocalVol_Forward_PDE_Solve("call",K,0,0,Tau,1.0);

print C;

