/*
**> In this program, we compute the local volatility from the implied volatility
** 
*/

new;
library option,pgraph;
OptionSet;

cls;

let Smile_K      = 80   85   90   95   100  105  110   115  120;
let Smile_ImpVol = 0.23 0.22 0.21 0.20 0.19 0.20 0.21 0.22 0.23;

S0 = 100;
r = 0.05;
b = 0.05;

declare matrix p_smoothing;

proc SigmaSurface(T,K);
 local r,c;
 r = _max_(rows(T),rows(K));
 c = _max_(cols(T),cols(K));
 retp(fspline(csspline(Smile_K,Smile_ImpVol,1,p_smoothing),K) .* ones(r,c));   /* Cubic Spline Smoothing */
endp;

p_smoothing = 0.10;
K = seqa(70,1,61);

T = seqa(0,0.01,101)';
ImpVol = SigmaSurface(t,K);
LocalVol = ImpVol_to_LocalVol(&SigmaSurface,0,0,0,T,K,S0,b);

pqgwin many;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  xtics(70,130,10,2);
  ytics(0,1,0.20,2);
  xlabel("\216K");
  ylabel("\206t\201");
  title("\216Implied Volatiliry (in %)");
  surface(K',T',100*ImpVol');

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  xtics(70,130,10,2);
  ytics(0,1,0.20,2);
  xlabel("\216K");
  ylabel("\206t\201");
  title("\216Local Volatiliry (in %)");
  surface(K',T',100*LocalVol');
