/*
**> Computing partial barrier option prices, Delta and Gamma
**
*/

new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

__output = 0;

S0 = 100;
sigma = 0.20;
tau = 1;
r = 0.08;
b = 0.03;

/*
**> The Partial Barrier has the following caracteristics:
**  
**    if 0.00 < t < 0.20 then 80 < S(t) < 120
**    if 0.20 < t < 0.50 then 70 < S(t)
**    if 0.50 < t < 0.70 then no barrier
**    if 0.70 < t < 0.90 then S(t) < 130
**    if t > 0.90 then 80 < S(t) < 115
**
*/

let Barrier[5,3] = 0.00 80 120 
                   0.2  70 .  
                   0.5  .  .  
                   0.7  .  130
                   0.9  80 115;

Barrier[.,1] = (0|73|182|255|328)/365;   // In fact, we consider the breaking dates in days
                                         // 73 days / 365 days ~ 0.20
                                         // 182 days / 365 days ~ 0.50
                                         // 255 days / 365 days ~ 0.70
                                         // 328 days / 365 days ~ 0.90
       
Smin = 0;
Smax = 200;
tmin = 0;
tmax = tau;

nS = 1001;
nt = 5001;

_PDE_SaveLastIter = 1;
_option_type = "call";

K = 85;
do until K > 110;

  call PartialBarrier_PDE(Smin,Smax,nS,tmin,tmax,nt,K,sigma,b,r,Barrier,"tmp",0.5);

  t = PDE_readFile("tmp","t");
  x = PDE_readFile("tmp","x");
  U = PDE_readFile("tmp","t"|t);

  indx = FindIndex(x,S0);
  S = x[indx];
  C = U[indx];

  {t_,x_,k_,h_,r_} = PDE_GetMesh(tmin,tmax,Nt,Smin,Smax,NS);

  Delta = (U[indx+2] - U[indx-2])/(4*h_);
  Gamma_ = (U[indx+1] + U[indx-1] - 2*C)/(h_^2); 

  print "----------------------------------------------";
  print ftos(K,         "Strike = %lf",3,2);
  print ftos(C,         "Price  = %lf",3,2);
  print ftos(100*Delta, "Delta  = %lf",3,2)$+" (in %)";
  print ftos(100*Gamma_,"Gamma  = %lf",3,2)$+" (in %)";

  K = K + 5;
endo;

 
