new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

declare matrix b,r,sigma,tau;

K = 100;

S0 = 80 | 90 | 100 | 110 | 120;
bVector = -0.04 | -0.04 | -0.04 | -0.04 ;
rVector =  0.08 | 0.12 | 0.08 | 0.08 ;
sigmaVector = 0.20 | 0.20 | 0.40 | 0.20 ;
tauVector = 0.25 | 0.25 | 0.25 | 0.50 ;
NtVector = floor(1825 * tauVector);
let string OptionType[2,1] = "call" "put";

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc bProc(t,x);
  retp( b .* x );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

/*
**> variational inequality
*/

proc eProc(t,x,U);
  local PayOff,cnd;
  
  if _Option_Type $== "call";
    PayOff = x - K;
    PayOff = PayOff .* (PayOff .> 0);
  else;
    PayOff = K - x;
    PayOff = PayOff .* (PayOff .> 0);
  endif;

  cnd = PayOff .> U;

  retp( cnd .* PayOff + (1-cnd) .* U );
endp;

proc tminBound(t,x);
  local PayOff;

  if _Option_Type $== "call";
    PayOff = x - K;
  else;
    PayOff = K - x;
  endif;

  PayOff = PayOff .* (PayOff .> 0);
  retp( PayOff );
endp;

proc xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc DxminBound(t,x);
  retp( -ones(rows(x),1) );
endp;

proc xmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;


_PDE_SaveLastIter = 1;
__output = 0;

i = 1;
do until i > 2;

  _Option_Type = OptionType[i];

  j = 1;
  do until j > 4;

    xmin = 50; xmax = 150; Nx = 501;
    tmin = 0; tmax = tauVector[j]; Nt = NtVector[j];

    b = bVector[j];
    r = rVector[j];
    sigma = sigmaVector[j];
    tau = tauVector[j];

    if _Option_Type $== "call";
      call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
                 &tminBound,&xminBound,0,0,&DxmaxBound);
    else;
      call PDE(&aProc,&bProc,&cProc,&dProc,&eProc,
                 &tminBound,0,&xmaxBound,&DxminBound,0);
    endif;

    call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);

    tau = PDE_readFile("pde","t");
    x = PDE_readFile("pde","x");
    U = PDE_readFile("pde","t"|tau);

    _fcmptol = 1e-12;  // Modify this value if you want that the procedure FindIndex works correctly

    indx = FindIndex(x,S0);
    x = x[indx];
    primeFD = U[indx];

    primeEU = EuropeanBS(x,K,sigma,tau,b,r);
    primeBAW = AmericanBS(x,K,sigma,tau,b,r);

    call PrintTable(x,K,sigma,tau,b,r,primeEU,primeBAW,primeFD,j);

    j = j + 1;

  endo;

  i = i + 1;
endo;

call DeleteFile("pde.dat");

proc (0) = PrintTable(x,K,sigma,tau,b,r,primeEU,primeBAW,primeFD,cn);
  local Nobs,e,mask,fmt,omat;

  Nobs = rows(x);
  e = ones(Nobs,1);
  K = K .* e;
  sigma = sigma .* e;
  tau = tau .* e;
  b = b .* e;
  r = r .* e;

  mask = 1;
  let fmt[9,3]=   "*.*lf" 7 1 "*.*lf" 7 1  "*.*lf" 7 2
                  "*.*lf" 7 2 "*.*lf" 7 2  "*.*lf" 8 2
                  "*.*lf" 11 3  "*.*lf" 11 3 "*.*lf" 11 3;

  omat = x~K~sigma~tau~b~r~primeEU~primeBAW~primeFD;

  if cn == 1;
    print /flush "    ";
    if _option_type $== "call";
      print "    S0     K     sigma   tau    b      r         Call       Call       Call     ";
    else;
      print "    S0     K     sigma   tau    b      r         Put        Put        Put      ";
    endif;
      print "                                                 European   American   American ";
      print "                                                            BAW        FD       ";
      print "================================================================================";
  endif;

  call printfm(omat,mask,fmt);
  retp;
endp;


