/*
**> In this example, we analyze the derivative d(ImpVol)/d(ATM) in the SABR model
**  for several values of beta
**
*/

new;

#define USE_OPTMUM

#ifdef USE_OPTMUM
  library option,pgraph,optmum;
#else
  library option,pgraph;
#endif

optionSet;
cls;
__output = 0;

let tau = 12;
tau = tau / 12;

// First, we transform the Normal SABR parametrization into Black volatilty smile

let params = 0.00895 0.00 0.57 0.23;
alpha = params[1]; beta = params[2]; nu = params[3]; rho = params[4];
F0 = 0.05;
let Strike = 0.03 0.04 0.05 0.06 0.07;
Smile = SABR_ImpVol_Normal(F0,Strike,alpha,beta,nu,rho,tau);
ImpVol = ImpVol_Normal_2_LN(F0,Strike,Smile,tau);                      // BS smile

let beta = 0 0.25 0.5 1;
K = seqa(0.01,0.09/100,101);
Smile1 = zeros(rows(K),rows(beta));
Smile2 = Smile1; NewSmile1 = Smile1; NewSmile2 = Smile2;
ATM = zeros(rows(beta),1); NewATM1 = ATM; NewATM2 = ATM;
alpha = zeros(rows(beta),1); nu = alpha; rho = alpha;

let shift = 0.01;

i = 1;
do until i > rows(beta);
  {alpha[i],beta[i],nu[i],rho[i]} = SABR_Calibrate(F0,Strike,ImpVol,beta[i],tau,0);
  ATM[i] = SABR_ATM(F0,alpha[i],beta[i],nu[i],rho[i],tau);
  Smile1[.,i] = SABR_ImpVol(F0,K,alpha[i],beta[i],nu[i],rho[i],tau);
  Smile2[.,i] = SABR_ImpVol_ATM(F0,K,ATM[i],beta[i],nu[i],rho[i],tau);
  NewATM1[i] = ATM[i] - shift;
  NewSmile1[.,i] = SABR_ImpVol_ATM(F0,K,NewATM1[i],beta[i],nu[i],rho[i],tau);
  NewATM2[i] = ATM[i] + shift;
  NewSmile2[.,i] = SABR_ImpVol_ATM(F0,K,NewATM2[i],beta[i],nu[i],rho[i],tau);
  i = i + 1;
endo;

graphset;
  makewind(9,6.855,0,0,1);
  makewind(4,3,4.5,3.5,1);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10;
  _pltype = 3|6|1|4;
  xlabel("\216Strike (in %)");
  ytics(0.7,1.4,0.1,2);

  setwind(1);
    let i = 1 2 3 4;
    _pline = 1~6~0~1~10~1~1~0~10;
    _plegstr = "\202b\201 = 0\000\202b\201 = 0.25\000\202b\201 = 0.5\000\202b\201 = 1";
    _plegctl = {2 5 1 1};
    xy(100*K,(NewSmile2[.,i]-Smile1[.,i])/shift);

  _plegctl = 0;

  setwind(2);
    _pnumht = 0.22; _paxht = 0.22; _ptitlht = 0.25;
    ylabel("\216Smile (in %)");
    ytics(15,40,5,2);
    xy(100*K,100*Smile1[.,1]);

  endwind;